@@ -1,6 +1,6 @@
 use std::{
     collections::{hash_map::Entry, HashMap},
-    env, mem,
+    env, iter, mem,
     sync::{Arc, Mutex},
 };
 
@@ -171,41 +171,35 @@ impl PointerDataExt for PointerData {
 #[macro_export]
 macro_rules! delegate_pointer {
     ($(@<$( $lt:tt $( : $clt:tt $(+ $dlt:tt )* )? ),+>)? $ty: ty) => {
-        $crate::reexports::client::delegate_dispatch!($(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty:
-            [
-                $crate::reexports::client::protocol::wl_pointer::WlPointer: $crate::seat::pointer::PointerData
-            ] => $crate::seat::SeatState
-        );
-        $crate::reexports::client::delegate_dispatch!($(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty:
-            [
-                $crate::reexports::protocols::wp::cursor_shape::v1::client::wp_cursor_shape_manager_v1::WpCursorShapeManagerV1: $crate::globals::GlobalData
-            ] => $crate::seat::pointer::cursor_shape::CursorShapeManager
-        );
-        $crate::reexports::client::delegate_dispatch!($(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty:
-            [
-                $crate::reexports::protocols::wp::cursor_shape::v1::client::wp_cursor_shape_device_v1::WpCursorShapeDeviceV1: $crate::globals::GlobalData
-            ] => $crate::seat::pointer::cursor_shape::CursorShapeManager
-        );
-     };
+        $crate::delegate_pointer!(@{ $(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty }; pointer: []);
+        $crate::delegate_pointer!(@{ $(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty }; pointer-only: $crate::seat::pointer::PointerData);
+    };
     ($(@<$( $lt:tt $( : $clt:tt $(+ $dlt:tt )* )? ),+>)? $ty: ty, pointer: [$($pointer_data:ty),* $(,)?]) => {
-        $crate::reexports::client::delegate_dispatch!($(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty:
-            [
-                $(
-                    $crate::reexports::client::protocol::wl_pointer::WlPointer: $pointer_data,
-                )*
-            ] => $crate::seat::SeatState
-        );
-        $crate::reexports::client::delegate_dispatch!($(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty:
+        $crate::delegate_pointer!(@{ $(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty }; pointer: [ $($pointer_data),* ]);
+    };
+    (@{$($ty:tt)*}; pointer: []) => {
+        $crate::reexports::client::delegate_dispatch!($($ty)*:
             [
                 $crate::reexports::protocols::wp::cursor_shape::v1::client::wp_cursor_shape_manager_v1::WpCursorShapeManagerV1: $crate::globals::GlobalData
             ] => $crate::seat::pointer::cursor_shape::CursorShapeManager
         );
-        $crate::reexports::client::delegate_dispatch!($(@< $( $lt $( : $clt $(+ $dlt )* )? ),+ >)? $ty:
+        $crate::reexports::client::delegate_dispatch!($($ty)*:
             [
                 $crate::reexports::protocols::wp::cursor_shape::v1::client::wp_cursor_shape_device_v1::WpCursorShapeDeviceV1: $crate::globals::GlobalData
             ] => $crate::seat::pointer::cursor_shape::CursorShapeManager
         );
     };
+    (@{$($ty:tt)*}; pointer-only: $pointer_data:ty) => {
+        $crate::reexports::client::delegate_dispatch!($($ty)*:
+            [
+                $crate::reexports::client::protocol::wl_pointer::WlPointer: $pointer_data
+            ] => $crate::seat::SeatState
+        );
+    };
+    (@$ty:tt; pointer: [$($pointer:ty),*]) => {
+        $crate::delegate_pointer!(@$ty; pointer: []);
+        $( $crate::delegate_pointer!(@$ty; pointer-only: $pointer); )*
+    }
 }
 
 #[derive(Debug, Default)]
@@ -464,31 +458,41 @@ impl<U: PointerDataExt + 'static, S: SurfaceDataExt + 'static> ThemedPointer<U,
         let mut themes = self.themes.lock().unwrap();
 
         let scale = self.surface.data::<S>().unwrap().surface_data().scale_factor();
-        let cursor = themes
-            .get_cursor(conn, icon.name(), scale as u32, &self.shm)
-            .map_err(PointerThemeError::InvalidId)?
-            .ok_or(PointerThemeError::CursorNotFound)?;
+        for cursor_icon_name in iter::once(&icon.name()).chain(icon.alt_names().iter()) {
+            if let Some(cursor) = themes
+                .get_cursor(conn, cursor_icon_name, scale as u32, &self.shm)
+                .map_err(PointerThemeError::InvalidId)?
+            {
+                let image = &cursor[0];
+                let (w, h) = image.dimensions();
+                let (hx, hy) = image.hotspot();
 
-        let image = &cursor[0];
-        let (w, h) = image.dimensions();
-        let (hx, hy) = image.hotspot();
+                self.surface.set_buffer_scale(scale);
+                self.surface.attach(Some(image), 0, 0);
 
-        self.surface.set_buffer_scale(scale);
-        self.surface.attach(Some(image), 0, 0);
+                if self.surface.version() >= 4 {
+                    self.surface.damage_buffer(0, 0, w as i32, h as i32);
+                } else {
+                    // Fallback for the old old surface.
+                    self.surface.damage(0, 0, w as i32 / scale, h as i32 / scale);
+                }
 
-        if self.surface.version() >= 4 {
-            self.surface.damage_buffer(0, 0, w as i32, h as i32);
-        } else {
-            // Fallback for the old old surface.
-            self.surface.damage(0, 0, w as i32 / scale, h as i32 / scale);
+                // Commit the surface to place the cursor image in the compositor's memory.
+                self.surface.commit();
+
+                // Set the pointer surface to change the pointer.
+                self.pointer.set_cursor(
+                    serial,
+                    Some(&self.surface),
+                    hx as i32 / scale,
+                    hy as i32 / scale,
+                );
+
+                return Ok(());
+            }
         }
 
-        // Commit the surface to place the cursor image in the compositor's memory.
-        self.surface.commit();
-
-        // Set the pointer surface to change the pointer.
-        self.pointer.set_cursor(serial, Some(&self.surface), hx as i32 / scale, hy as i32 / scale);
-        Ok(())
+        Err(PointerThemeError::CursorNotFound)
     }
 
     /// Hide the cursor by providing empty surface for it.
